/***************************************************************
**  TestRNG.c	Test file for Intel Random Number Generator
**		Must Have Intel RNG compatible motherboard and
**		Processor.  Requires Intel Crypto Provider
**		Header File icsp4ms.h at www.intel.com
**
**		Written by Brett Bartholomew
**		Visit www.bartdart.com for great freeware
****************************************************************/
#define WIN32_LEAN_AND_MEAN		// Tricking WINVER for Crypto API
#define _WIN32_WINNT 0x0400
#define WINVER 0x0400

//Includes
#include <windows.h>
#include <stdio.h>
#include <wincrypt.h>	//Windows Crypto API
#include "icsp4ms.h"	//Intel Crypto Provider (www.intel.com)

//Functions	
bool GetRandomChunk(int iSize, BYTE* pbBuffer);
void PrintError();

//Entry Point
int main(int argc, char* argv[])
{
	
	BYTE* pbRandoms = new BYTE[0];
	if (GetRandomChunk(100,pbRandoms))
		printf("%u",pbRandoms);
	else
	{
		printf("Error returned from RNG:\n");
		PrintError();
	}	
	delete pbRandoms;
	return 0;
}

//**************************************************************************
//GetRandomChunk --Connects to HW Random Number Generator and fills
//BYTE buffer pbBuffer with iSize number of Randoms
//**************************************************************************
bool GetRandomChunk(int iSize, BYTE* pbBuffer)
{
	
	//Crypto Provider
	HCRYPTPROV	hProv;

	// Get a handle to the Intel CSP.
	if(!CryptAcquireContext(&hProv, NULL, INTEL_DEF_PROV, PROV_INTEL_SEC, 0))
	{    
		return false;
	}
	
	//Generate the RNs
	if (CryptGenRandom(hProv,iSize,pbBuffer) != TRUE)
	{    
		// Release the CSP handle    
		CryptReleaseContext(hProv, 0); 
		return false;
	}

	// Release the CSP handle
	CryptReleaseContext(hProv, 0);

	return true;
}


//**************************************************************************
//GetErrorText --Formats a Message from GetLastError and prints it
//**************************************************************************
void PrintError()
{

	LPVOID lpMsgBuf;
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
	);

	// Display the string.
	printf("%s",(LPSTR)lpMsgBuf);

	// Free the buffer.
	LocalFree( lpMsgBuf );
}
